using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Provider;
using Windows.Storage.Streams;
using Windows.UI;
using Windows.UI.Popups;

using WinRT.Interop;

using Microsoft.UI;
using Microsoft.UI.Text;
using Microsoft.UI.Xaml;
using Microsoft.UI.Xaml.Controls;
using Microsoft.UI.Xaml.Media;
using Microsoft.UI.Xaml.Shapes;

namespace TestProject
{
    /// <summary>
    ///  
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Ȱ  ϱ - GetActiveWindow()

        /// <summary>
        /// Ȱ  ϱ
        /// </summary>
        /// <returns></returns>
        [DllImport("user32", ExactSpelling = true, CharSet = CharSet.Auto, PreserveSig = true, SetLastError = false)]
        private static extern IntPtr GetActiveWindow();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        ///  
        /// </summary>
        private Color currentColor = Colors.Green;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - MainPage()

        /// <summary>
        /// 
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            this.openFileButton.Click    += openFileButton_Click;
            this.saveFileButton.Click    += saveFileButton_Click;
            this.boldButton.Click        += boldButton_Click;
            this.italicButton.Click      += italicButton_Click;
            this.redColorButton.Click    += colorButton_Click;
            this.orangeColorButton.Click += colorButton_Click;
            this.yellowColorButton.Click += colorButton_Click;
            this.greenColorButton.Click  += colorButton_Click;
            this.blueColorButton.Click   += colorButton_Click;
            this.indigoColorButton.Click += colorButton_Click;
            this.violetColorButton.Click += colorButton_Click;
            this.grayColorButton.Click   += colorButton_Click;
            this.richEditBox.GotFocus    += richEditBox_GotFocus;
            this.richEditBox.TextChanged += richEditBox_TextChanged;

            this.richEditBox.Document.GetDefaultCharacterFormat().ForegroundColor = Colors.Green;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region Open file ư Ŭ óϱ - openFileButton_Click(sender, e)

        /// <summary>
        /// Open file ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private async void openFileButton_Click(object sender, RoutedEventArgs e)
        {
            FileOpenPicker fileOpenPicker = new FileOpenPicker();

            fileOpenPicker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary;

            fileOpenPicker.FileTypeFilter.Add(".rtf");

            if(Window.Current == null)
            {
                IntPtr windowHandle = GetActiveWindow();

                InitializeWithWindow.Initialize(fileOpenPicker, windowHandle);
            }

            StorageFile storageFile = await fileOpenPicker.PickSingleFileAsync();

            if(storageFile != null)
            {
                using(IRandomAccessStream stream = await storageFile.OpenAsync(FileAccessMode.Read))
                {
                    this.richEditBox.Document.LoadFromStream(TextSetOptions.FormatRtf, stream);
                }
            }
        }

        #endregion
        #region Save file ư Ŭ óϱ - saveFileButton_Click(sender, e)

        /// <summary>
        /// Save file ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private async void saveFileButton_Click(object sender, RoutedEventArgs e)
        {
            FileSavePicker fileSavePicker = new FileSavePicker
            {
                SuggestedStartLocation = PickerLocationId.DocumentsLibrary
            };

            fileSavePicker.FileTypeChoices.Add("Rich Text", new List<string>() { ".rtf" });

            fileSavePicker.SuggestedFileName = "New Document";

            if(Window.Current == null)
            {
                IntPtr windowHandle = GetActiveWindow();

                InitializeWithWindow.Initialize(fileSavePicker, windowHandle);
            }

            StorageFile storageFile = await fileSavePicker.PickSaveFileAsync();

            if(storageFile != null)
            {
                CachedFileManager.DeferUpdates(storageFile);

                using(IRandomAccessStream stream = await storageFile.OpenAsync(FileAccessMode.ReadWrite))
                {
                    this.richEditBox.Document.SaveToStream(TextGetOptions.FormatRtf, stream);
                }

                FileUpdateStatus status = await CachedFileManager.CompleteUpdatesAsync(storageFile);

                if(status != FileUpdateStatus.Complete)
                {
                    MessageDialog messageDialog = new MessageDialog("File " + storageFile.Name + " couldn't be saved.");

                    await messageDialog.ShowAsync();
                }
            }
        }

        #endregion
        #region Bold ư Ŭ óϱ - boldButton_Click(sender, e)

        /// <summary>
        /// Bold ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void boldButton_Click(object sender, RoutedEventArgs e)
        {
            this.richEditBox.Document.Selection.CharacterFormat.Bold = FormatEffect.Toggle;
        }

        #endregion
        #region Italic ư Ŭ óϱ - italicButton_Click(sender, e)

        /// <summary>
        /// Italic ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void italicButton_Click(object sender, RoutedEventArgs e)
        {
            this.richEditBox.Document.Selection.CharacterFormat.Italic = FormatEffect.Toggle;
        }

        #endregion
        #region  ư Ŭ óϱ - colorButton_Click(sender, e)

        /// <summary>
        ///  ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void colorButton_Click(object sender, RoutedEventArgs e)
        {
            Button button = sender as Button;

            Rectangle rectangle = button.Content as Rectangle;

            Color color = (rectangle.Fill as SolidColorBrush).Color;

            this.richEditBox.Document.Selection.CharacterFormat.ForegroundColor = color;

            this.fontColorButton.Flyout.Hide();

            this.richEditBox.Focus(FocusState.Keyboard);

            this.currentColor = color;
        }

        #endregion
        #region ġ Ʈ ڽ Ŀ ȹ óϱ - richEditBox_GotFocus(sender, e)

        /// <summary>
        /// ġ Ʈ ڽ Ŀ ȹ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void richEditBox_GotFocus(object sender, RoutedEventArgs e)
        {
            this.richEditBox.Document.GetText(TextGetOptions.UseCrlf, out _);
            
            ITextRange textRange = richEditBox.Document.GetRange(0, TextConstants.MaxUnitCount);

            SolidColorBrush backgroundBrush = App.Current.Resources["TextControlBackgroundFocused"] as SolidColorBrush;

            if(backgroundBrush != null)
            {
                textRange.CharacterFormat.BackgroundColor = backgroundBrush.Color;
            }
        }

        #endregion
        #region ġ Ʈ ڽ ؽƮ  óϱ - richEditBox_TextChanged(sender, e)

        /// <summary>
        /// ġ Ʈ ڽ ؽƮ  óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void richEditBox_TextChanged(object sender, RoutedEventArgs e)
        {
            if(this.richEditBox.Document.Selection.CharacterFormat.ForegroundColor != this.currentColor)
            {
                this.richEditBox.Document.Selection.CharacterFormat.ForegroundColor = this.currentColor;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region Find ؽƮ ڽ ̶Ʈ ϱ - SetFindTextBoxHighlight()

        /// <summary>
        /// Find ؽƮ ڽ ̶Ʈ ϱ
        /// </summary>
        private void SetFindTextBoxHighlight()
        {
            ClearFindTextBoxHighlight();

            Color highlightBackgroundColor = (Color)App.Current.Resources["SystemColorHighlightColor"    ];
            Color highlightForegroundColor = (Color)App.Current.Resources["SystemColorHighlightTextColor"];

            string textToFind = this.findTextBox.Text;

            if(textToFind != null)
            {
                ITextRange textRange = this.richEditBox.Document.GetRange(0, 0);

                while(textRange.FindText(textToFind, TextConstants.MaxUnitCount, FindOptions.None) > 0)
                {
                    textRange.CharacterFormat.BackgroundColor = highlightBackgroundColor;
                    textRange.CharacterFormat.ForegroundColor = highlightForegroundColor;
                }
            }
        }

        #endregion
        #region Find ؽƮ ڽ ̶Ʈ  - ClearFindTextBoxHighlight()

        /// <summary>
        /// Find ؽƮ ڽ ̶Ʈ 
        /// </summary>
        private void ClearFindTextBoxHighlight()
        {
            ITextRange textRange = this.richEditBox.Document.GetRange(0, TextConstants.MaxUnitCount);

            SolidColorBrush defaultBackgroundBrush = this.richEditBox.Background as SolidColorBrush;
            SolidColorBrush defaultForegroundBrush = this.richEditBox.Foreground as SolidColorBrush;

            textRange.CharacterFormat.BackgroundColor = defaultBackgroundBrush.Color;
            textRange.CharacterFormat.ForegroundColor = defaultForegroundBrush.Color;
        }

        #endregion
    }
}